@@ -674,8 +674,10 @@ socket_connected (SoupSocket *sock, GSocketConnection *conn, GError *error)
 
 		g_object_unref (priv->connect_cancel);
 		priv->connect_cancel = NULL;
-		if (cancelled)
+		if (cancelled) {
+			g_clear_error (&error);
 			return SOUP_STATUS_CANCELLED;
+		}
 	}
 
 	if (error) {
@@ -1563,7 +1565,7 @@ soup_socket_read_until (SoupSocket *sock, gpointer buffer, gsize len,
 			SOUP_FILTER_INPUT_STREAM (priv->istream),
 			buffer, len, boundary, boundary_len,
 			!priv->non_blocking,
-			got_boundary, cancellable, &my_err);
+			TRUE, got_boundary, cancellable, &my_err);
 		status = translate_read_status (sock, cancellable,
 						my_nread, nread, my_err, error);
 	}
