@@ -8,6 +8,12 @@
 #include <string.h>
 #include <unistd.h>
 
+#define LIBSOUP_USE_UNSTABLE_REQUEST_API
+#include <libsoup/soup-multipart-input-stream.h>
+#include <libsoup/soup-request-http.h>
+#include <libsoup/soup-requester.h>
+#include <libsoup/soup.h>
+
 #include "test-utils.h"
 
 #define READ_BUFFER_SIZE 8192
@@ -20,7 +26,7 @@ typedef enum {
 } MultipartMode;
 
 char *buffer;
-SoupSession *session;
+SoupRequester *requester;
 char *base_uri_string;
 SoupURI *base_uri;
 SoupMultipartInputStream *multipart;
@@ -87,8 +93,10 @@ content_sniffed (SoupMessage *msg, char *content_type, GHashTable *params, int *
 static void
 check_is_next (gboolean is_next)
 {
-	soup_test_assert (is_next,
-			  "expected a header, but there are no more headers");
+	if (!is_next) {
+		debug_printf (1, "  expected a header, but there are no more headers\n");
+		errors++;
+	}
 }
 
 static void
@@ -110,8 +118,15 @@ got_headers (SoupMessage *msg, int *headers_count)
 		check_is_next (is_next);
 	}
 
-	g_assert_cmpstr (name, ==, "Content-Type");
-	g_assert_cmpstr (value, ==, "multipart/x-mixed-replace; boundary=cut-here");
+	if (!g_str_equal (name, "Content-Type")) {
+		debug_printf (1, "  expected `Content-Type' got %s\n", name);
+		errors++;
+	}
+
+	if (!g_str_equal (value, "multipart/x-mixed-replace; boundary=cut-here")) {
+		debug_printf (1, "  expected `multipart/x-mixed-replace; boundary=cut-here' got %s\n", value);
+		errors++;
+	}
 }
 
 static void
@@ -120,20 +135,23 @@ read_cb (GObject *source, GAsyncResult *asyncResult, gpointer data)
 	GMainLoop *loop = (GMainLoop*)data;
 	GInputStream *stream = G_INPUT_STREAM (source);
 	GError *error = NULL;
-	gssize bytes_read;
-
-	bytes_read = g_input_stream_read_finish (stream, asyncResult, &error);
-	g_assert_no_error (error);
+	gssize bytes_read = g_input_stream_read_finish (stream, asyncResult, &error);
 	if (error) {
-		g_object_unref (stream);
+		debug_printf (1, "  failed read: %s\n", error->message);
+		errors++;
+
 		g_main_loop_quit (loop);
 		return;
 	}
 
 	if (!bytes_read) {
 		g_input_stream_close (stream, NULL, &error);
-		g_assert_no_error (error);
-		g_object_unref (stream);
+
+		if (error) {
+			debug_printf (1, "  failed close: %s\n", error->message);
+			errors++;
+		}
+
 		g_main_loop_quit (loop);
 		return;
 	}
@@ -152,8 +170,11 @@ no_multipart_handling_cb (GObject *source, GAsyncResult *res, gpointer data)
 	GInputStream* in;
 
 	in = soup_request_send_finish (request, res, &error);
-	g_assert_no_error (error);
+
 	if (error) {
+		debug_printf (1, "  failed send: %s\n", error->message);
+		errors++;
+
 		g_main_loop_quit (loop);
 		return;
 	}
@@ -170,7 +191,10 @@ multipart_close_part_cb (GObject *source, GAsyncResult *res, gpointer data)
 	GError *error = NULL;
 
 	g_input_stream_close_finish (in, res, &error);
-	g_assert_no_error (error);
+	if (error) {
+		debug_printf (1, "  error closing stream: %s\n", error->message);
+		errors++;
+	}
 }
 
 static void multipart_next_part_cb (GObject *source,
@@ -182,20 +206,32 @@ check_read (gsize nread, unsigned passes)
 {
 	switch (passes) {
 	case 0:
-		g_assert_cmpint (nread, ==, 30);
+		if (nread != 30) {
+			debug_printf (1, "  expected to read 30 bytes, got: %d\n", (int)nread);
+			errors++;
+		}
 		break;
 	case 1:
-		g_assert_cmpint (nread, ==, 10);
+		if (nread != 10) {
+			debug_printf (1, "  expected to read 10 bytes, got: %d\n", (int)nread);
+			errors++;
+		}
 		break;
 	case 2:
-		g_assert_cmpint (nread, ==, 24);
+		if (nread != 24) {
+			debug_printf (1, "  expected to read 24 bytes, got: %d\n", (int)nread);
+			errors++;
+		}
 		break;
 	case 3:
-		g_assert_cmpint (nread, ==, 34);
+		if (nread != 34) {
+			debug_printf (1, "  expected to read 34 bytes, got: %d\n", (int)nread);
+			errors++;
+		}
 		break;
 	default:
-		soup_test_assert (FALSE, "unexpected read of size: %d", (int)nread);
-		break;
+		debug_printf (1, "  unexpected read of size: %d\n", (int)nread);
+		errors++;
 	}
 }
 
@@ -209,11 +245,13 @@ multipart_read_cb (GObject *source, GAsyncResult *asyncResult, gpointer data)
 	gssize bytes_read;
 
 	bytes_read = g_input_stream_read_finish (in, asyncResult, &error);
-	g_assert_no_error (error);
+
 	if (error) {
+		debug_printf (1, "  failed read: %s\n", error->message);
+		errors++;
+
 		g_input_stream_close_async (in, G_PRIORITY_DEFAULT, NULL,
 					    multipart_close_part_cb, NULL);
-		g_object_unref (in);
 
 		g_main_loop_quit (loop);
 		return;
@@ -227,7 +265,6 @@ multipart_read_cb (GObject *source, GAsyncResult *asyncResult, gpointer data)
 
 		g_input_stream_close_async (in, G_PRIORITY_DEFAULT, NULL,
 					    multipart_close_part_cb, NULL);
-		g_object_unref (in);
 
 		soup_multipart_input_stream_next_part_async (multipart, G_PRIORITY_DEFAULT, NULL,
 							     multipart_next_part_cb, data);
@@ -256,22 +293,43 @@ check_headers (SoupMultipartInputStream* multipart, unsigned passes)
 		is_next = soup_message_headers_iter_next (&iter, &name, &value);
 		check_is_next (is_next);
 
-		g_assert_cmpstr (name, ==, "Content-Type");
-		g_assert_cmpstr (value, ==, "text/html");
+		if (!g_str_equal (name, "Content-Type")) {
+			debug_printf (1, "  [0] expected `Content-Type' got %s\n", name);
+			errors++;
+		}
+
+		if (!g_str_equal (value, "text/html")) {
+			debug_printf (1, "  [0] expected `text/html' got %s\n", value);
+			errors++;
+		}
 
 		is_next = soup_message_headers_iter_next (&iter, &name, &value);
 		check_is_next (is_next);
 
-		g_assert_cmpstr (name, ==, "Content-Length");
-		g_assert_cmpstr (value, ==, "30");
+		if (!g_str_equal (name, "Content-Length")) {
+			debug_printf (1, "  [0] expected `Content-Length' got %s\n", name);
+			errors++;
+		}
+
+		if (!g_str_equal (value, "30")) {
+			debug_printf (1, "  [0] expected `30' got %s\n", value);
+			errors++;
+		}
 
 		break;
 	case 1:
 		is_next = soup_message_headers_iter_next (&iter, &name, &value);
 		check_is_next (is_next);
 
-		g_assert_cmpstr (name, ==, "Content-Length");
-		g_assert_cmpstr (value, ==, "10");
+		if (!g_str_equal (name, "Content-Length")) {
+			debug_printf (1, "  [1] expected `Content-Length' got %s\n", name);
+			errors++;
+		}
+
+		if (!g_str_equal (value, "10")) {
+			debug_printf (1, "  [1] expected `10' got %s\n", value);
+			errors++;
+		}
 
 		break;
 	case 2:
@@ -279,12 +337,19 @@ check_headers (SoupMultipartInputStream* multipart, unsigned passes)
 		is_next = soup_message_headers_iter_next (&iter, &name, &value);
 		check_is_next (is_next);
 
-		g_assert_cmpstr (name, ==, "Content-Type");
-		g_assert_cmpstr (value, ==, "text/css");
+		if (!g_str_equal (name, "Content-Type")) {
+			debug_printf (1, "  [%d] expected `Content-Type' got %s\n", passes, name);
+			errors++;
+		}
+
+		if (!g_str_equal (value, "text/css")) {
+			debug_printf (1, "  [%d] expected `text/html' got %s\n", passes, value);
+			errors++;
+		}
 
 		break;
 	default:
-		soup_test_assert (FALSE, "unexpected part received");
+		debug_printf (1, "  unexpected part received\n");
 		break;
 	}
 }
@@ -300,17 +365,21 @@ multipart_next_part_cb (GObject *source, GAsyncResult *res, gpointer data)
 	g_assert (SOUP_MULTIPART_INPUT_STREAM (source) == multipart);
 
 	in = soup_multipart_input_stream_next_part_finish (multipart, res, &error);
-	g_assert_no_error (error);
+
 	if (error) {
-		g_clear_error (&error);
-		g_object_unref (multipart);
+		debug_printf (1, "  failed next part: %s\n", error->message);
+		errors++;
+
 		g_main_loop_quit (loop);
 		return;
 	}
 
 	if (!in) {
-		g_assert_cmpint (passes, ==, 4);
-		g_object_unref (multipart);
+		if (passes != 4) {
+			debug_printf (1, "  expected 4 parts, got %u\n", passes);
+			errors++;
+		}
+
 		g_main_loop_quit (loop);
 		return;
 	}
@@ -335,17 +404,17 @@ multipart_handling_cb (GObject *source, GAsyncResult *res, gpointer data)
 	SoupMessage *message;
 
 	in = soup_request_send_finish (request, res, &error);
-	g_assert_no_error (error);
+	message = soup_request_http_get_message (SOUP_REQUEST_HTTP (request));
+	multipart = soup_multipart_input_stream_new (message, in);
+
 	if (error) {
+		debug_printf (1, "  failed send: %s\n", error->message);
+		errors++;
+
 		g_main_loop_quit (loop);
 		return;
 	}
 
-	message = soup_request_http_get_message (SOUP_REQUEST_HTTP (request));
-	multipart = soup_multipart_input_stream_new (message, in);
-	g_object_unref (message);
-	g_object_unref (in);
-
 	if (g_object_get_data (source, "multipart-small-reads"))
 		g_object_set_data (G_OBJECT (multipart), "multipart-small-reads", GINT_TO_POINTER(1));
 
@@ -365,21 +434,23 @@ sync_multipart_handling_cb (GObject *source, GAsyncResult *res, gpointer data)
 	gsize bytes_read;
 
 	in = soup_request_send_finish (request, res, &error);
-	g_assert_no_error (error);
+	message = soup_request_http_get_message (SOUP_REQUEST_HTTP (request));
+	multipart = soup_multipart_input_stream_new (message, in);
+
 	if (error) {
+		debug_printf (1, "  failed send: %s\n", error->message);
+		errors++;
+
 		g_main_loop_quit (loop);
 		return;
 	}
 
-	message = soup_request_http_get_message (SOUP_REQUEST_HTTP (request));
-	multipart = soup_multipart_input_stream_new (message, in);
-	g_object_unref (message);
-	g_object_unref (in);
-
 	while (TRUE) {
 		in = soup_multipart_input_stream_next_part (multipart, NULL, &error);
-		g_assert_no_error (error);
+
 		if (error) {
+			debug_printf (1, "  failed sync next part: %s\n", error->message);
+			errors++;
 			g_clear_error (&error);
 			break;
 		}
@@ -390,46 +461,56 @@ sync_multipart_handling_cb (GObject *source, GAsyncResult *res, gpointer data)
 		check_headers (multipart, passes);
 
 		g_input_stream_read_all (in, (void*)buffer, sizeof (buffer), &bytes_read, NULL, &error);
-		g_assert_no_error (error);
+
 		if (error) {
+			debug_printf (1, "  failed sync read: %s\n", error->message);
+			errors++;
 			g_clear_error (&error);
-			g_object_unref (in);
 			break;
 		}
 
 		check_read (bytes_read, passes);
 
 		passes++;
-		g_object_unref (in);
 	}
 
-	g_assert_cmpint (passes, ==, 4);
+	if (passes != 4) {
+		debug_printf (1, "  expected 4 parts, got %u\n", passes);
+		errors++;
+	}
 
 	g_main_loop_quit (loop);
-	g_object_unref (multipart);
+
+}
+
+static const char*
+multipart_mode_to_string (MultipartMode mode)
+{
+	if (mode == NO_MULTIPART)
+		return "NO_MULTIPART";
+	else if (mode == SYNC_MULTIPART)
+		return "SYNC_MULTIPART";
+	else if (mode == ASYNC_MULTIPART_SMALL_READS)
+		return "SYNC_MULTIPART_SMALL_READS";
+
+	return "ASYNC_MULTIPART";
 }
 
 static void
-test_multipart (gconstpointer data)
+test_multipart (int headers_expected, int sniffed_expected, MultipartMode multipart_mode)
 {
-	int headers_expected = 1, sniffed_expected = 1;
-	MultipartMode multipart_mode = GPOINTER_TO_INT (data);
-	SoupRequest* request;
-	SoupMessage *msg;
-	GMainLoop *loop;
+	GError* error = NULL;
+	SoupRequest* request = soup_requester_request (requester, base_uri_string, &error);
+
+	SoupMessage *msg = soup_request_http_get_message (SOUP_REQUEST_HTTP (request));
+	GMainLoop *loop = g_main_loop_new (NULL, TRUE);
 	int headers_count = 0;
 	int sniffed_count = 0;
 	GHashTable *params;
 	const char *content_type;
 	gboolean message_is_multipart = FALSE;
-	GError* error = NULL;
 
-	request = soup_session_request (session, base_uri_string, &error);
-	g_assert_no_error (error);
-	if (error)
-		return;
-
-	msg = soup_request_http_get_message (SOUP_REQUEST_HTTP (request));
+	debug_printf (1, "test_multipart(%s)\n", multipart_mode_to_string (multipart_mode));
 
 	/* This is used to track the number of parts. */
 	passes = 0;
@@ -444,7 +525,7 @@ test_multipart (gconstpointer data)
 	g_signal_connect (msg, "content-sniffed",
 			  G_CALLBACK (content_sniffed), &sniffed_count);
 
-	loop = g_main_loop_new (NULL, TRUE);
+	g_object_ref (msg);
 
 	if (multipart_mode == ASYNC_MULTIPART)
 		soup_request_send_async (request, NULL, multipart_handling_cb, loop);
@@ -467,9 +548,25 @@ test_multipart (gconstpointer data)
 	}
 	g_clear_pointer (&params, g_hash_table_unref);
 
-	g_assert_true (message_is_multipart);
-	g_assert_cmpint (headers_count, ==, headers_expected);
-	g_assert_cmpint (sniffed_count, ==, sniffed_expected);
+	if (!message_is_multipart) {
+		debug_printf (1,
+			      "	 Header does not indicate a multipart message!\n");
+		errors++;
+	}
+
+	if (headers_count != headers_expected) {
+		debug_printf (1,
+			      "	 expected got_header %d times, got %d!\n",
+			      headers_expected, headers_count);
+		errors++;
+	}
+
+	if (sniffed_count != sniffed_expected) {
+		debug_printf (1,
+			      "	 expected content_sniffed %d times, got %d!\n",
+			      sniffed_expected, sniffed_count);
+		errors++;
+	}
 
 	g_object_unref (msg);
 	g_object_unref (request);
@@ -480,15 +577,16 @@ int
 main (int argc, char **argv)
 {
 	SoupServer *server;
-	int ret;
+	SoupSession *session;
 
 	test_init (argc, argv, NULL);
 
 	buffer = g_malloc (READ_BUFFER_SIZE);
 
-	server = soup_test_server_new (SOUP_TEST_SERVER_DEFAULT);
+	server = soup_test_server_new (FALSE);
 	soup_server_add_handler (server, NULL, server_callback, NULL, NULL);
-	base_uri = soup_test_server_get_uri (server, "http", NULL);
+	base_uri = soup_uri_new ("http://127.0.0.1");
+	soup_uri_set_port (base_uri, soup_server_get_port (server));
 	base_uri_string = soup_uri_to_string (base_uri, FALSE);
 
 	/* FIXME: I had to raise the number of connections allowed here, otherwise I
@@ -499,22 +597,21 @@ main (int argc, char **argv)
 					 "max-conns", 20,
 					 "max-conns-per-host", 20,
 					 NULL);
+	soup_session_add_feature_by_type (session, SOUP_TYPE_REQUESTER);
 	soup_session_add_feature_by_type (session, SOUP_TYPE_CONTENT_SNIFFER);
 
-	g_test_add_data_func ("/multipart/no", GINT_TO_POINTER (NO_MULTIPART), test_multipart);
-	g_test_add_data_func ("/multipart/sync", GINT_TO_POINTER (SYNC_MULTIPART), test_multipart);
-	g_test_add_data_func ("/multipart/async", GINT_TO_POINTER (ASYNC_MULTIPART), test_multipart);
-	g_test_add_data_func ("/multipart/async-small-reads", GINT_TO_POINTER (ASYNC_MULTIPART_SMALL_READS), test_multipart);
+	requester = SOUP_REQUESTER (soup_session_get_feature (session, SOUP_TYPE_REQUESTER));
 
-	ret = g_test_run ();
+	test_multipart (1, 1, NO_MULTIPART);
+	test_multipart (1, 1, SYNC_MULTIPART);
+	test_multipart (1, 1, ASYNC_MULTIPART);
+	test_multipart (1, 1, ASYNC_MULTIPART_SMALL_READS);
 
 	soup_uri_free (base_uri);
 	g_free (base_uri_string);
-	g_free (buffer);
 
 	soup_test_session_abort_unref (session);
 	soup_test_server_quit_unref (server);
-
 	test_cleanup ();
-	return ret;
+	return errors != 0;
 }
